/**
 * Created by nl12 on 15/03/2016.
 */
"use strict"
function Chart(canvas, maxPoints) {
    this.canvas = canvas;
    this.ctx = canvas.getContext("2d");
    this.maxPoints = maxPoints;
    this.max = 0; //Used for auto scaling
    this.min = 0;
    this.scaling = 1;
    this.fontSize = canvas.height / 20;
    this.running = false;
    this.plots = {};

    this.clear = () => {
        this.ctx.clearRect(0, 0, this.canvas.width, this.canvas.height);
    }

    this.start = ()=> {
        this.running = true;
        this.render();
    }

    this.stop = ()=> {
        this.running = false;
    }

    this.render = ()=> {
        if (!this.running) {
            return;
        }
        this.clear();

        for (let key in this.plots) {
            let plot = this.plots[key];
            this.ctx.beginPath();
            const ps = plot.points.map((elem, i)=> {
                return {
                    x: this.canvas.width / this.maxPoints * i,
                    y: canvas.height - (elem - this.min) * this.scaling
                }
            });
            this.ctx.moveTo(ps[0].x, ps[0].y);

            ps.forEach(p=>this.ctx.lineTo(p.x, p.y));

            this.ctx.strokeStyle = plot.col;
            this.ctx.stroke();
        }
        this.ctx.font = this.fontSize + "px serif"
        for (let i = 0; i < 6; i++) {
            this.ctx.fillText(((this.max - (this.max - this.min) / 5 * i)).toFixed(0), 0, this.canvas.height / 5 * i + this.fontSize);
        }
        this.ctx.closePath();

        window.requestAnimationFrame(this.render);
    }
    this.addPlot = (name, col)=> {
        this.plots[name] = {
            name: name,
            col: col,
            points: []
        }
        for (let i = 0; i < this.maxPoints; i++) {
            this.plots[name].points[i] = 0;
        }
    }
    this.randomDarkColour = ()=>{
        let col = "#";
        for(let i=0;i<6;i++){
            col += Math.floor(Math.random()*10);
        }
        return col;
    };

    this.append = (point,name)=> {
		name = name || "default";
        if(!this.plots[name]){
            this.addPlot(name,this.randomDarkColour());
        }
        const points = this.plots[name].points;
        points.push(point);
        points.shift();
        const allPoints = Object.keys(this.plots)
            .map(x=>this.plots[x].points)
            .reduce((p,c)=>p.concat(c));
        this.max = Math.max(...allPoints);
        this.min = Math.min(...allPoints);
        this.scaling = this.canvas.height / (this.max - this.min);
    }
    this.start();
}
